# Overview

This is an experimental repo for testing the FIST client.  All components for fist clients should be added here.  The goal is to eventually update the svelte-pi library to also support fist apps.

To run

    go to root directory
    npm install
    npm run dev
    go to http://localhost:3000/fist.html


### Design Notes


Requirements

* Easy to edit/modify/customize
* Support DBT-client
* (Mostly) compatible with client-lib design 
* Responsive to window resizing
* Aligns Charts and widget sizes and positioning

#### Resizing

* Each chart and widget calls `getContext("dims")` to know the widget and margins it should use
* Any container, or even the top level app during setup, can call `setContext("dims", dims)` to adjust the dimensions of its descendant components.
* This ensures components within a Container are consistently sized and aligned
* See [Container.svelte](./Container.svelte) to see this in action.

`dims` is a writable store with the following example initialization:

```ts
dims = writable({
  w:600,     // pixel width
  h:300,     // pixel height
  margin: {
    left:10,
    right:10,
    top:10,
    bottom:10
  }
})
```

#### Components

Each component is expected to  use `getContext("dims")` and size itself relative to it.
Otherwise, they should adhere to the contracts dictated by `svelte-pi`.   Specifically:

* Widgets that users interact with export `addSourceIact(iact)` which registers an interaction
* Views that render data export `render(data, name)` where `data` is a table to be rendered, and `name` is an optional value passed by a widget.  It's mainly used to create overlays.

#### Backends

The high level relationship between backends, views, and widgets can be found in [the root README](../../README.md)

To summarize, a backend exports

* `on(name, callback)` to listen 
* `emit(name, ...args)` which it calls when there's data available
* `async execute(query, ...args)` where `query` is _any_ representation of a request (a sql string, parameter values, etc) that the backend recognizes, and `...args` is any other arguments passed in by the caller.   
  * It should call `emit("data", result, ...args)` once the request results are recieved
  * It should return the result


Overall design to get dbt-client to work

* Create a custom dbt backend for each dbt model, 
  * `execute()` expects a dictionary with the model's variables as keys
  * backend keeps track of the current variables' values
  * backend generates SQL that's sent to duckdb for processing
* App will 
  * create and load a single duckdb backend with all the data it needs
  * create a dbt backend for each dbt model
  * hook up widgets, charts, and backends as followis

```
in general:

  dbt backend i --> chart i
  widgets --> dbt backend i

for overlays:
  overlay backend i --> chart i, 
  widgets --> overlay backend i
```
