import * as R from "ramda";
import { get } from "svelte/store";
import { backendLib, viewLib } from "../stores";
import { project } from "../util";
import { InteractionSpec, isExecutable, isManipulable } from "./spec";
export default class Interaction {
    id;
    source;
    target;
    m; // manipulation spec
    type;
    space;
    constructor(spec) {
        let _viewLib = get(viewLib);
        let _backendLib = get(backendLib);
        this.id = spec.id;
        this.m = spec.m;
        this.type = this.m?.type;
        this.space = this.m?.space;
        this.postprocess = spec.postprocess ?? R.identity;
        this.source = isManipulable(spec.source) ? spec.source : _viewLib[spec.source];
        this.target = isExecutable(spec.target) ? spec.target : _backendLib[spec.target];
        //console.log("Interaction.new", spec, this.source, this.target)
        if (isManipulable(this.source))
            this.source.addSourceIact(this);
        else
            console.warn("Interaction source doesn't define addSourceIact", this.source);
    }
    // called by a source (View) when the user interacts with it.
    //
    // @param data a table containing the interaction state
    trigger(data, ...args) {
        let binding = this.postprocess(data);
        this.target.execute(binding, ...args);
    }
    preview(data) {
        throw new Error("Not Implemented");
    }
}
